#version 130
#extension GL_EXT_gpu_shader4 : enable

#define WATER_REFLECTION
uniform sampler2DArray texture;
#ifdef WATER_REFLECTION
uniform sampler2D reflectionmap;
#endif
uniform sampler2D underwatermap;

uniform int texrange;
uniform vec3 focal;
uniform vec3 screendata;
uniform vec4 lightVector;
uniform float daylight;
uniform float visibility;

in vec3 texCoord;
in vec4 colordata;
in vec4 torchlight;

in float vertdist;
in vec3 viewVector;
in float amplitude;
in float modulation;

const float FOG_DENSITY   = 0.5;
const float FOG_LUMINANCE = 0.5;
const float FOG_MAXIMUM   = 0.2;

const float SHADOW_EXP = 0.9;
const float SHADOW_MUL = 1.2;
const float SHADOW_MAX = 0.98;

const float SPECULARNESS     = 1.0;

#define NEAR_CLAMP 0.3   // opaque-ness at maximal distance
#define FAR_CLAMP  1.0   // alpha at near distance

#ifdef FOG_LUM
vec4 hjemmebakt_fog(vec4 color) {
	float f = min(FOG_MAXIMUM, pow(vertdist / focal[2] * 1.05, 1.25) * FOG_DENSITY);
	float lum = min(1.0, (0.7 + 0.2126 * color.r + 0.7152 * color.g + 0.0722 * color.b) * FOG_LUMINANCE);
	return vec4(mix(color.rgb, vec3(lum), f), color[3]);
}
#endif

void main(void) {

	if (vertdist > focal[2]) discard; //ze end of ze world!
	
	float expdist = min(1.0, pow(vertdist / focal[2], 0.5));
	
	vec4 color;
	
if (texrange == 8) {

	color = vec4(0.11, 0.33, 1.1, 0.6); // texture2DArray(texture, texCoord.xyz);
	vec4 refcolor;
	
	// make water less penetratable when darkened
	// color[3] = max(mix(color[3], 1.0, alpha*alpha), color[3]);
	
	// blend less the further away from camera
	color[3] = clamp(expdist, NEAR_CLAMP, FAR_CLAMP);
	// blend with underground
	vec2 refcoord = vec2( min(1.0, (gl_FragCoord.x + amplitude *  1.5) / screendata[0]) , 
								   (gl_FragCoord.y - amplitude * 12.0) / screendata[1]);
	color = mix(texture2D(underwatermap, refcoord.xy), color, color[3]);
	
	// specular from sun
	float s = max(0.0, dot(viewVector, reflect(lightVector.xyz, vec3(0.0, 0.6667, 0.0)) ) );
	color.rgb += s * s * s * SPECULARNESS;
	
	// ocean waves
	color.rgb *= 1.0 + pow(max(0.0, amplitude), 3.0) * 3.0;
	
#ifdef WATER_REFLECTION
	refcoord = vec2( gl_FragCoord.x / screendata[0] , (gl_FragCoord.y - amplitude * 12.0) / screendata[1] );
	refcolor = texture2D(reflectionmap, refcoord.xy);
	color.rgb = mix(color.rgb, refcolor.rgb, expdist * 0.75);
#endif
	
} else {
	// lava, fog etc.
	color = texture2DArray(texture, texCoord.xyz);
}
	
	// upramp shadow gamma
	float shadow = 1.0 - min(SHADOW_MAX, pow(colordata[0], 0.9) );
	
	float brightness = colordata[1]; // torchlight brightness
	
	// apply scaled lighting
	color.rgb *= min( 1.0, smoothstep(0.0, 1.0, daylight * shadow + brightness ) );
	
	// mix in torchlight
	color.rgb = mix( color.rgb, torchlight.rgb, torchlight[3] * (1.0 - brightness + modulation) );
	
	// finalize color
	color = vec4(color.rgb, visibility);
	
	// world fade-out
	float fadedist = focal[2] * 0.88;
	if (vertdist > fadedist) {
		float fstr = (vertdist - fadedist) / (focal[2] - fadedist);
		color.rgb *= 1.0 + fstr;
		color[3] *= 1.0 - fstr;
	}
	
#ifdef FOG_LUM
	gl_FragColor = hjemmebakt_fog(color);
#else
	gl_FragColor = color;
#endif
}
